package com.biganto.visual.roompark.domain.use_case

import com.biganto.visual.roompark.data.repository.db.requrey.model.EstateEntity
import com.biganto.visual.roompark.domain.contract.AuthContract
import com.biganto.visual.roompark.domain.contract.DealContract
import com.biganto.visual.roompark.domain.contract.FlatPlanContract
import com.biganto.visual.roompark.domain.model.EstateModel
import com.biganto.visual.roompark.domain.model.fromEntity
import io.reactivex.Observable
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 24.09.2019.
 */

class  EstateUseCase @Inject constructor(
    private val contract: DealContract,
    private val planContract: FlatPlanContract,
    private val authContract: AuthContract
) {


    fun prefetchFavorites() =
        authContract.currentUser()
            .map {user -> user.estates?.asSequence()
                ?.map { it as EstateEntity }
                ?.filter{ it.favorite }?.filterNotNull()?.toList()
            }
            .map { fromEntity(it, ::fromEntity) }

    fun fetchFavorites(): Observable<List<EstateModel>> =
        authContract.currentUser()
            .flatMap (contract::fetchFavorites)
            .map { fromEntity(it, ::fromEntity) }

    fun getEstate(estateId: Int) = contract.getEstate(estateId)

    fun fetchEstate(building:Int, number:Int) =
        contract.fetchEstate(building, number)

    fun getEstatePlanPresets(estateId: Int) =
        planContract.getPlanTypes(estateId)


    fun getPlan(
        estateId: Int
        , planId: Int
        , furniture: Boolean?
        , sizes: Boolean?
        , walls: Boolean?
        , electric: Boolean?
    ) = planContract.getPlan(
        estateId
        , planId
        , furniture
        , sizes
        , walls
        , electric
    )

}